<?php

namespace Fastrack\Utilities\Seekom;


use League\OAuth2\Client\Token\AccessToken;
use League\OAuth2\Client\Token\AccessTokenInterface;
use SilverStripe\Core\Environment;

/**
 * @property string $clientId
 * @property string $clientSecret
 * @property string $authUrl
 * @property string $apiUrl
 * @property string $accessToken
 * @property string $tokenType
 * @property string $expiresIn
 * @property string $refreshToken
 * @property string $refreshTokenExpiresIn
 */
class SeekomCredentials
{

	private array $data;

	public function __construct(array $data)
	{
		$this->data = $data;
	}

	public function getData(): array
	{
		return $this->data;
	}

	public static function getVaultPath(): string
	{
		$vaultClient = Environment::getEnv('VAULT_CLIENT');
		$isUat = Environment::getEnv('SEEKOM_IS_UAT') === 'true';
		return $vaultClient . '/seekom/' . ($isUat ? 'uat' : 'live');
	}

	public function getAccessToken(): AccessTokenInterface
	{
		return new AccessToken([
			'access_token' => $this->accessToken,
			'token_type' => $this->tokenType,
			'expires_in' => intval($this->expiresIn),
			'refresh_token' => $this->refreshToken,
			'refresh_token_expires_in' => intval($this->refreshTokenExpiresIn),
		]);
	}

	public function setAccessToken(AccessTokenInterface $token)
	{
		$values = $token->getValues();
		$this->data['accessToken'] = $token->getToken();
		$this->data['expiresIn'] = strval($token->getExpires());
		if ($token->getRefreshToken()) {
			$this->data['refreshToken'] = $token->getRefreshToken();
		}
		if (array_key_exists('token_type', $values) && $values['token_type']) {
			$this->data['tokenType'] = $values['token_type'];
		}
		if (array_key_exists('refresh_token_expires_in', $values) && $values['refresh_token_expires_in']) {
			$this->data['refreshTokenExpiresIn'] = strval($values['refresh_token_expires_in']);
		}
	}

	public function hasAccessToken(): bool
	{
		try {
			$accessToken = $this->getAccessToken();
			return $accessToken->getRefreshToken() && $accessToken->getToken();
		} catch (\InvalidArgumentException $e) {
			return false;
		}
	}

	public function __set($property, $value)
	{
		return $this->data[$property] = $value;
	}

	public function __get($property)
	{
		if (array_key_exists($property, $this->data)) {
			return $this->data[$property];
		}
		return null;
	}

}
