<?php

namespace Fastrack\Utilities;

use Exception;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Http\Message\ResponseInterface;
use SilverStripe\Core\Environment;
use SilverStripe\SiteConfig\SiteConfig;

class Vault
{

	private ClientInterface $client;
	private string $vaultToken;
	private string $vaultUrl;

	public function __construct(ClientInterface $client)
	{
		$this->client = $client;
		$this->vaultUrl = Environment::getEnv('VAULT_URL');
		$this->vaultToken = Environment::getEnv('VAULT_TOKEN');
	}

	/**
	 * @param string $secret
	 * @param string $path
	 * @return mixed
	 * @throws GuzzleException
	 * @throws Exception
	 */
	public function getSecret(string $secret, string $path): array
	{
		$url = $this->vaultUrl . "/v1/{$secret}/data/{$path}";
		$response = $this->client->request('GET', $url, [
			'headers' => [
				'X-Vault-Request' => 'true',
				'X-Vault-Token' => $this->vaultToken,
			]
		]);
		if ($response->getStatusCode() === 200) {
			$body = json_decode($response->getBody(), true);
			return $body['data']['data'];
		}
		throw new Exception($this->handleErrors($response));
	}

	/**
	 * @param string $secret
	 * @param string $path
	 * @param $value
	 * @throws GuzzleException|Exception
	 */
	public function setSecret(string $secret, string $path, $value)
	{
		$url = $this->vaultUrl . "/v1/{$secret}/data/{$path}";
		$response = $this->client->request('POST', $url, [
			'headers' => [
				'X-Vault-Request' => 'true',
				'X-Vault-Token' => $this->vaultToken,
			],
			'json' => ['data' => $value],
		]);
		if ($response->getReasonPhrase() !== 'OK') {
			throw new Exception($this->handleErrors($response));
		}
	}

	private function handleErrors(ResponseInterface $response): string
	{
		$message = 'An unexpected error occurred';
		$body = $response->getBody();
		if ($body && $body['errors']) {
			$message = join(', ', $body['errors']);
		}
		return $message;
	}

}
