<?php


namespace Fastrack\Utilities\Tasks;


use SebastianBergmann\CodeCoverage\Report\PHP;
use SilverStripe\Assets\File;
use SilverStripe\Dev\BuildTask;

class ImportSeo extends BuildTask
{

	public function __construct()
	{
		parent::__construct();
	}

	public function getTitle()
	{
		return 'Import SEO meta data from csv file';
	}

	public function getDescription()
	{
		return 'Columns order: slug, meta title, meta description. Do not include column titles. ' .
			'Use ?file=Uploads/seo.csv. Where seo.csv is a file you have uploaded to the CMS. Remember ' .
			'to delete the file after import';
	}

	/**
	 * @param \SilverStripe\Control\HTTPRequest $request
	 */
	public function run($request)
	{
		$file = $this->getFile($request->getVar('file'));
		$stream = $file->getStream();
		$pagesPublished = [];
		$pagesUpdated = [];
		$pagesNotFound = [];

		while (($data = fgetcsv($stream, 1000, ",")) !== false) {
			$page = \Page::get_by_link($data[0]);
			if (!$page) {
				$pagesNotFound[] = $data[0];
				continue;
			}
			$isPublished = $page->isPublished();
			$page->MetaTitle = $data[1];
			$page->MetaDescription = $data[2];
			$page->write();
			if ($isPublished) {
				$page->publishSingle();
				$pagesPublished[] = $data[0];
			} else {
				$pagesUpdated[] = $data[0];
			}
		}

		$this->printResult($pagesPublished, 'published');
		$this->printResult($pagesUpdated, 'updated');
		$this->printResult($pagesNotFound, 'not found');

	}

	private function getFile($fileName)
	{
		$file = File::find($fileName);
		if (!$file) {
			throw new \Exception('Could not find file. Ensure to pass "file" as parameter e.g. file=Uploads/seo.csv');
		}
		if ($file->getExtension() !== 'csv') {
			throw new \Exception('File not csv');
		}
		return $file;
	}

	private function printResult($result, $name)
	{
		if (count($result)) {
			echo "<h2>Pages {$name}</h2>" . PHP_EOL;
			echo '<ul>' . PHP_EOL;
			foreach ($result as $page) {
				echo "<li>{$page}</li>" . PHP_EOL;
			}
			echo '</ul>' . PHP_EOL;
		}
	}


}
